@@ -66,6 +66,9 @@ typedef struct sz_implementations_t {
 
     sz_utf8_case_fold_t utf8_case_fold;
     sz_utf8_case_insensitive_find_t utf8_case_insensitive_find;
+
+    sz_utf8_word_find_boundary_t utf8_word_find_boundary;
+    sz_utf8_word_rfind_boundary_t utf8_word_rfind_boundary;
     sz_utf8_case_insensitive_order_t utf8_case_insensitive_order;
 
     sz_sequence_argsort_t sequence_argsort;
@@ -117,6 +120,9 @@ static void sz_dispatch_table_update_implementation_(sz_capability_t caps) {
 
     impl->utf8_case_fold = sz_utf8_case_fold_serial;
     impl->utf8_case_insensitive_find = sz_utf8_case_insensitive_find_serial;
+
+    impl->utf8_word_find_boundary = sz_utf8_word_find_boundary_serial;
+    impl->utf8_word_rfind_boundary = sz_utf8_word_rfind_boundary_serial;
     impl->utf8_case_insensitive_order = sz_utf8_case_insensitive_order_serial;
 
     impl->sequence_argsort = sz_sequence_argsort_serial;
@@ -422,7 +428,7 @@ SZ_DYNAMIC void sz_sha256_state_update(sz_sha256_state_t *state, sz_cptr_t data,
     sz_dispatch_table.sha256_state_update(state, data, length);
 }
 
-SZ_DYNAMIC void sz_sha256_state_digest(sz_sha256_state_t const *state, sz_u8_t *digest) {
+SZ_DYNAMIC void sz_sha256_state_digest(sz_sha256_state_t const *state, sz_u8_t digest[sz_at_least_(32)]) {
     sz_dispatch_table.sha256_state_digest(state, digest);
 }
 
@@ -446,7 +452,7 @@ SZ_DYNAMIC void sz_fill(sz_ptr_t target, sz_size_t length, sz_u8_t value) {
     sz_dispatch_table.fill(target, length, value);
 }
 
-SZ_DYNAMIC void sz_lookup(sz_ptr_t target, sz_size_t length, sz_cptr_t source, sz_cptr_t lut) {
+SZ_DYNAMIC void sz_lookup(sz_ptr_t target, sz_size_t length, sz_cptr_t source, char const lut[sz_at_least_(256)]) {
     sz_dispatch_table.lookup(target, length, source, lut);
 }
 
@@ -517,9 +523,18 @@ SZ_DYNAMIC sz_size_t sz_utf8_case_fold(sz_cptr_t source, sz_size_t source_length
 
 SZ_DYNAMIC sz_cptr_t sz_utf8_case_insensitive_find( //
     sz_cptr_t haystack, sz_size_t haystack_length,  //
-    sz_cptr_t needle, sz_size_t needle_length, sz_size_t *matched_length) {
+    sz_cptr_t needle, sz_size_t needle_length,      //
+    sz_utf8_case_insensitive_needle_metadata_t *needle_metadata, sz_size_t *matched_length) {
     return sz_dispatch_table.utf8_case_insensitive_find(haystack, haystack_length, needle, needle_length,
-                                                        matched_length);
+                                                        needle_metadata, matched_length);
+}
+
+SZ_DYNAMIC sz_cptr_t sz_utf8_word_find_boundary(sz_cptr_t text, sz_size_t length, sz_size_t *boundary_width) {
+    return sz_dispatch_table.utf8_word_find_boundary(text, length, boundary_width);
+}
+
+SZ_DYNAMIC sz_cptr_t sz_utf8_word_rfind_boundary(sz_cptr_t text, sz_size_t length, sz_size_t *boundary_width) {
+    return sz_dispatch_table.utf8_word_rfind_boundary(text, length, boundary_width);
 }
 
 SZ_DYNAMIC sz_ordering_t sz_utf8_case_insensitive_order( //
